#!/usr/local/bin/python3

import subprocess
import pandas as pd
import statistics
import numpy as np

#https://fish-evol.org/GMT_ji.html

# タブ区切りのテキストファイルを読み込み、データフレームに格納
file_white_plots = '150_lat_lon_white_all.txt'
file_red_plots = '150_lat_lon_red_selected.txt'
#file_white_plots = '150_cruseAll_lat_lon_kawakami.txt'
#file_red_plots = '150_sampleID_lat_lon_kawakami.txt'
#file_white_plots = '150_cruseAll_lat_lon_northSea.txt'
#file_red_plots = '150_sampleID_lat_lon_northSea.txt'
#file_white_plots = '150_cruseAll_lat_lon_Miya22.txt'
#file_red_plots = '150_sampleID_lat_lon_Miya22.txt'
#file_white_plots = '150_cruseAll_lat_lon_kuroshio.txt'
#file_red_plots = '150_sampleID_lat_lon_kuroshio.txt'
#file_white_plots = '150_cruseAll_lat_lon_3c.txt'
#file_red_plots = '150_sampleID_lat_lon_3c.txt'
#file_white_plots = '150_cruseAll_lat_lon_2c.txt'
#file_red_plots = '150_sampleID_lat_lon_2c.txt'
#file_white_plots = '150_cruseAll_lat_lon_kawakami_south.txt'
#file_red_plots = '150_sampleID_lat_lon_kawakami_south.txt'
#file_white_plots = '150_cruseAll_lat_lon_kawakami_equator.txt'
#file_red_plots = '150_sampleID_lat_lon_kawakami_equator.txt'

eachDirAddress = "./"

print("file_white_plots", file_white_plots)

################
# Calculate the average Longitude considering the wrap-around at 180 degrees
def calculate_average_longitude(df):
    # Convert longitudes to radians
    df['Longitude_rad'] = df['Longitude'].apply(lambda x: x * (np.pi / 180.0))
    
    # Calculate the average of cosines and sines of longitudes
    avg_cos = df['Longitude_rad'].apply(lambda x: np.cos(x)).mean()
    avg_sin = df['Longitude_rad'].apply(lambda x: np.sin(x)).mean()
    
    # Calculate the average longitude in radians
    avg_longitude_rad = np.arctan2(avg_sin, avg_cos)
    
    # Convert the average longitude back to degrees
    avg_longitude_deg = avg_longitude_rad * (180.0 / np.pi)
    
    return avg_longitude_deg

# Function to calculate west_longitude and east_longitude
def calculate_west_east_longitude(longitude, value_to_west, value_to_east):
    # Convert longitude and latitude to radians
    longitude_rad = longitude * (np.pi / 180.0)
    
    # Calculate west_longitude and east_longitude in radians
    west_longitude_rad = longitude_rad - value_to_west * (np.pi / 180.0)
    east_longitude_rad = longitude_rad + value_to_east * (np.pi / 180.0)
    
    # Convert back to degrees
    west_longitude_deg = west_longitude_rad * (180.0 / np.pi)
    east_longitude_deg = east_longitude_rad * (180.0 / np.pi)
    
    return west_longitude_deg, east_longitude_deg

###############


df = pd.read_csv(file_white_plots, delimiter='\t', header=None, names=['Longitude', 'Latitude', 'Station'])

# Latitude 列から数値以外の値を除外する
df = df[pd.to_numeric(df['Latitude'], errors='coerce').notnull()]

# Latitude 列あるいは Longitude 列が数値以外の行を除外する
df['Latitude'] = pd.to_numeric(df['Latitude'])
df['Longitude'] = pd.to_numeric(df['Longitude'])

# Latitude 列と Longitude 列を数値に変換する
df['Latitude'] = pd.to_numeric(df['Latitude'])
df['Longitude'] = pd.to_numeric(df['Longitude'])

# 値が 0 の行を除外する
df = df[(df['Latitude'] != 0) & (df['Longitude'] != 0)]

################################################
### latitude
#mean_latitude = df['Latitude'].mean()

#median_latitude = 0
##median_latitude = statistics.median(df['Latitude'])
## 中央値を計算する（中央値候補の片方を選ぶ）
#sorted_latitudes = sorted(df['Latitude'])
#n = len(sorted_latitudes)
#if n % 2 == 0:
#    median_latitude = sorted_latitudes[n // 2 - 1]  # 中央値候補の片方を選ぶ
#else:
#    median_latitude = sorted_latitudes[n // 2]


################################################
### longitude
#list_longitudes = []
#for longitude in df['Longitude']:
#    #print("longitude1", longitude)
#    #if float(longitude) < -90 :
#    if longitude < -90 :
#        longitude = 360 + longitude
#        #print("longitude2", longitude)
#    #print("")
#    #list_longitudes.append(float(longitude))
#    list_longitudes.append(longitude)

#mean_longitude = sum(list_longitudes) / len(list_longitudes)

# 中央値を計算する（中央値候補の片方を選ぶ
#median_longitude = statistics.median(list_longitudes)
#sorted_longitude = sorted(df['Longitude'])
#n = len(sorted_longitude)
#if n % 2 == 0:
#    median_longitude = sorted_longitude[n // 2 - 1]  # 中央値候補の片方を選ぶ
#else:
#    median_longitude = sorted_longitude[n // 2]

# 経度をラジアンに変換し、180度線を考慮して中央値を計算する関数
def calculate_median_longitude(df):
    longitudes = df['Longitude'].apply(lambda x: x if x >= 0 else x + 360)
    median_longitude = np.median(longitudes)
    median_longitude = median_longitude if median_longitude <= 180 else median_longitude - 360
    return median_longitude

# 経度の中央値を計算
median_longitude = calculate_median_longitude(df)


#### longitude の range を計算する
# Function to calculate the range of longitude considering the wrap-around at 180 degrees
def calculate_range_longitude(df):
    longitudes = df['Longitude'].apply(lambda x: x if x >= 0 else x + 360)
    range_longitude = longitudes.max() - longitudes.min()
    return range_longitude

# Calculate the range of longitude in degrees
range_longitude_deg = calculate_range_longitude(df)


################################################
# 中心となる station の選択

# Calculate the average Longitude
mean_trig_longitude = calculate_average_longitude(df)

# Find the Station with the closest Latitude to the average Longitude
selected_station = df.iloc[(df['Longitude'] - mean_trig_longitude).abs().argmin()]['Station']
print("selected_station", selected_station)

selected_longitude = df[df['Station'] == selected_station]['Longitude'].values[0]
selected_latitude = df[df['Station'] == selected_station]['Latitude'].values[0]

################################################


print("df")
print(df)

print("selected_station   :", selected_station)
print("selected_longitude :", selected_longitude)
print("selected_latitude  :", selected_latitude)

#print("mean_latitude      :", mean_latitude)
#print("median_latitude    :", median_latitude)

#print("mean_longitude     :", mean_longitude)
print("median_longitude   :", median_longitude)
print("range_longitude_deg    :", range_longitude_deg)

#exit()
################################################

#exit()



if 50 <= range_longitude_deg:
    map_type  = "Large-type"
elif 16 <= range_longitude_deg and range_longitude_deg < 50:
    map_type  = "Mediam-type"
else:
    map_type  = "Small-type"

if map_type == "Large-type":
    psbasemap_B = "a10f10g10"
    #west_tip = selected_longitude - 90
    #east_tip = selected_longitude + 90
    # Calculate west_longitude and east_longitude for St10
    value_to_west = 90
    value_to_east = 90
    west_tip, east_tip = calculate_west_east_longitude(selected_longitude, value_to_west, value_to_east)

    if 70 < selected_latitude:
        map_type_A = "Large-type_over70"
        print("## 1 ##")
        north_tip = selected_latitude + 5
        south_tip = selected_latitude - 45
    elif 58 < selected_latitude and selected_latitude <= 70:
        map_type_A = "Large-type_58_70"
        #print("## 1 ##")
        north_tip = selected_latitude + 20
        south_tip = selected_latitude - 35
    elif 20 < selected_latitude and selected_latitude <= 58:
        map_type_A = "Large-type_20_58"
        north_tip = selected_latitude + 30
        south_tip = selected_latitude - 25

    elif -20 < selected_latitude and selected_latitude <= 20:
        map_type_A = "Large-type_-20_20"
        north_tip = selected_latitude + 45
        south_tip = selected_latitude - 45

    elif -58 < selected_latitude and selected_latitude <= -20:
        map_type_A = "Large-type_-58_-20"
        north_tip = selected_latitude + 25
        south_tip = selected_latitude - 25

    elif -70 < selected_latitude and selected_latitude <= -58:
        map_type_A = "Large-type_under-58"
        north_tip = selected_latitude + 35
        south_tip = selected_latitude - 20
    elif -90 < selected_latitude and selected_latitude <= -70:
        map_type_A = "Large-type_under-70"
        #print("## 1 ##")
        north_tip = selected_latitude + 45
        south_tip = selected_latitude - 5
    else:
        print("Error: check latitude", selected_latitude)
        exit()

elif map_type == "Mediam-type":
    psbasemap_B = "a5f6g5"
    map_type_A = "Mediam-typ_A"
    north_tip = selected_latitude + 17
    south_tip = selected_latitude - 17
    west_tip = selected_longitude - 25
    east_tip = selected_longitude + 25
else:
    psbasemap_B = "a1f1g1"
    map_type_A = "Small_A"
    north_tip = selected_latitude + 6
    south_tip = selected_latitude - 5
    west_tip = selected_longitude - 8
    east_tip = selected_longitude + 9

print("############## map_type_A #############")
print(map_type_A)
print("#######################################")

R_grdimage_region = str(west_tip) + "/" + str(east_tip) + "/" + str(south_tip) + "/" + str(north_tip)  # "110/168/-56/-22"

#R_grdimage_region = "124/140/25/36" #"125/139/26/35"
#R_grdimage_region = "125/139/26/35"
print("R_grdimage_region", R_grdimage_region)
#exit()

############################


line_rmdir = "rm -r outfiles_py"
#print("line_rmdir", line_rmdir, "<br>")
subprocess.call(line_rmdir, shell=True)

line_mkdir = "mkdir outfiles_py"
#print("line_mkdir", line_mkdir, "<br>")
subprocess.call(line_mkdir, shell=True)


#gmt makecpt -Chaxby -T-6000/0/10 -Z > outfiles_py/050_haxby_area.cpt
line_makecpt = "gmt makecpt -Chaxby -T-6000/0/10 -Z > outfiles_py/050_haxby_area.cpt"
#print("line_makecpt", line_makecpt, "<br>")
subprocess.call(line_makecpt, shell=True)

#gmt grdimage ~/ETOPO1_Ice_g_gmt4.grd -R125/139/26/35 -JM24c  -Coutfiles/050_haxby_area.cpt -X1c -Y1c -K >> outfiles_py/150map.ps
#gmt grdimage ~/ETOPO1_Ice_g_gmt4.grd -R125/139/-35/-26 -JM24c  -Coutfiles/050_haxby_area.cpt -X1c -Y1c -K >> outfiles_py/150map.ps
#line_grdimage = "gmt grdimage ~/ETOPO1_Ice_g_gmt4.grd -R125/139/26/35 -JM24c  -Coutfiles_py/050_haxby_area.cpt -X1c -Y1c -K >> outfiles_py/150map.ps"
line_grdimage = f"gmt grdimage ~/ETOPO1_Ice_g_gmt4.grd -R{R_grdimage_region} -JM24c  -Coutfiles_py/050_haxby_area.cpt -X1c -Y1c -K >> outfiles_py/150map.ps"
#print("line_grdimage", line_grdimage, "<br>")
subprocess.call(line_grdimage, shell=True)

#gmt grdcontour ~/ETOPO1_Ice_g_gmt4.grd -R125/139/26/35 -JM24c -C500 -A500 -L-6000/-0 -W0.1 -K -O -V >> outfiles_py/150map.ps
#gmt grdcontour ~/ETOPO1_Ice_g_gmt4.grd -R125/139/-35/-26 -JM24c -C500 -A500 -L-6000/-0 -W0.1 -K -O -V >> outfiles_py/150map.ps
#line_grdcontour = "gmt grdcontour ~/ETOPO1_Ice_g_gmt4.grd -R125/139/26/35 -JM24c -C500 -A500 -L-6000/-0 -W0.1 -K -O -V >> outfiles_py/150map.ps"
line_grdcontour = f"gmt grdcontour ~/ETOPO1_Ice_g_gmt4.grd -R{R_grdimage_region} -JM24c -C500 -A500 -L-6000/-0 -W0.1 -K -O -V >> outfiles_py/150map.ps"
print("line_grdcontour", line_grdcontour, "<br>")
subprocess.call(line_grdcontour, shell=True)
#exit()

#gmt pscoast -R125/139/26/35 -JM24c -Di -Ggray -W0.1,black -K -O -V >> outfiles_py/150map.ps
#gmt pscoast -R125/139/-35/-26  -JM24c -Di -Ggray -W0.1,black -K -O -V >> outfiles_py/150map.ps
#line_pscoast = "gmt pscoast -R125/139/26/35 -JM24c -Di -Ggray -W0.1,black -K -O -V >> outfiles_py/150map.ps"
line_pscoast = f"gmt pscoast -R{R_grdimage_region} -JM24c -Di -Ggray -W0.1,black -K -O -V >> outfiles_py/150map.ps"
#print("line_pscoast", line_pscoast, "<br>")
subprocess.call(line_pscoast, shell=True)

#gmt psscale -D24.5/5/10/0.5 -Coutfiles/050_haxby_area.cpt -Bf1000a2000 -K -O -V >> outfiles_py/150map.ps
line_psscale = "gmt psscale -D24.5/5/10/0.5 -Coutfiles_py/050_haxby_area.cpt -Bf1000a2000 -K -O -V >> outfiles_py/150map.ps"
print("line_psscale", line_psscale, "<br>")
subprocess.call(line_psscale, shell=True)

#gmt psbasemap -R125/139/26/35 -JM24c -Ba1f1g1 -BWSne+tKuroshio-region -K -O -V >> outfiles_py/150map.ps
#gmt psbasemap -R125/139/-35/-26 -JM24c -Ba1f1g1 -BWSne+tKuroshio-region -K -O -V >> outfiles_py/150map.ps
#line_psbasemap = "gmt psbasemap -R125/139/26/35 -JM24c -Ba1f1g1 -BWSne+tKuroshio-region -K -O -V >> outfiles_py/150map.ps"

#if map_type == "wide":
#    #psbasemap_B = "a10f10g10"
#    line_psbasemap = "gmt psbasemap -R" + R_grdimage_region + " -JM24c -Ba10f10g10 -BWSne+tLarge-type -K -O -V >> outfiles_py/150map.ps"
#else:
#    #psbasemap_B = "a1f1g1"
#    line_psbasemap = "gmt psbasemap -R" + R_grdimage_region + " -JM24c -Ba1f1g1 -BWSne+tLarge-type -K -O -V >> outfiles_py/150map.ps"
line_psbasemap = f"gmt psbasemap -R{R_grdimage_region} -JM24c -B{psbasemap_B} -BWSne+t{map_type} -K -O -V >> outfiles_py/150map.ps"

#print("line_psbasemap", line_psbasemap, "<br>")
subprocess.call(line_psbasemap, shell=True)
#exit()

#psxy 1
#gmt psxy 150_cruseAll_lat_lon.txt -R -JM -Sc0.40 -W1 -Gwhite -K -O -V >> outfiles_py/150map.ps
line_psxy = f"gmt psxy {file_white_plots} -R -JM -Sc0.40 -W1 -Gwhite -K -O -V >> outfiles_py/150map.ps"
print("line_psxy", line_psxy, "<br>")
subprocess.call(line_psxy, shell=True)

#gmt pstext 150_cruseAll_lat_lon.txt -R125/139/26/35 -JM24c -F+f12p,Helvetica,-=0.5p,black+jBL -K -O -V >> outfiles_py/150map.ps
#gmt pstext 150_cruseAll_lat_lon.txt -R125/139/-35/-26 -JM24c -F+f12p,Helvetica,-=0.5p,black+jBL -K -O -V >> outfiles_py/150map.ps
#line_pstext = "gmt pstext 150_cruseAll_lat_lon.txt -R125/139/26/35 -JM24c -F+f12p,Helvetica,-=0.5p,black+jBL -K -O -V >> outfiles_py/150map.ps"
line_pstext = f"gmt pstext {file_white_plots} -R" + R_grdimage_region + " -JM24c -F+f12p,Helvetica,-=0.5p,black+jBL -K -O -V >> outfiles_py/150map.ps"
#print("line_pstext", line_pstext, "<br>")
subprocess.call(line_pstext, shell=True)

#psxy 2
#gmt psxy 150_sampleID_lat_lon.txt -R -JM -Sc0.40 -W1 -Gred -O -V >> outfiles_py/150map.ps
line_psxy = f"gmt psxy {file_red_plots} -R -JM -Sc0.40 -W1 -Gred -O -V >> outfiles_py/150map.ps"
#print("line_psxy", line_psxy, "<br>")
subprocess.call(line_psxy, shell=True)

#psconvert1
#gmt psconvert outfiles_py/150map.ps -P -Tg -E200 -V
line_psconvert = "gmt psconvert outfiles_py/150map.ps -P -Tg -E200 -V"
#print("line_psconvert", line_psconvert, "<br>")
subprocess.call(line_psconvert, shell=True)

#psconvert2
#gmt psconvert outfiles_py/150map.ps -P -Tf -E200
line_psconvert = "gmt psconvert outfiles_py/150map.ps -P -Tf -E200"
#print("line_psconvert", line_psconvert, "<br>")
subprocess.call(line_psconvert, shell=True)

print("map_type_A", map_type_A)